@@ -186,8 +186,10 @@ class RequestDialog(Factory.Popup):
         help_texts = self.app.wallet.get_help_texts_for_receive_request(req)
         address = req.get_address() or ''
         URI = self.app.wallet.get_request_URI(req) or ''
-        lnaddr = req.lightning_invoice or ''
-        self.status = self.app.wallet.get_request_status(self.key)
+        lnaddr = ""
+        if req.is_lightning():
+            lnaddr = self.app.wallet.get_bolt11_invoice(req)
+        self.status = self.app.wallet.get_invoice_status(req)
         self.status_str = req.get_status_str(self.status)
         self.status_color = pr_color[self.status]
         self.has_lightning = req.is_lightning()
