@@ -6,8 +6,9 @@ from electrum import ecc
 from electrum.i18n import _
 from electrum.util import UserCancelled
 from electrum.keystore import bip39_normalize_passphrase
-from electrum.bip32 import BIP32Node, convert_bip32_path_to_list_of_uint32
+from electrum.bip32 import BIP32Node, convert_bip32_strpath_to_intpath
 from electrum.logging import Logger
+from electrum.plugin import runs_in_hwd_thread
 from electrum.plugins.hw_wallet.plugin import HardwareClientBase, HardwareHandlerBase
 
 
@@ -131,6 +132,7 @@ class SafeTClientBase(HardwareClientBase, GuiMixin, Logger):
     def is_pairable(self):
         return not self.features.bootloader_mode
 
+    @runs_in_hwd_thread
     def has_usable_connection_with_device(self):
         try:
             res = self.ping("electrum pinging device")
@@ -145,6 +147,7 @@ class SafeTClientBase(HardwareClientBase, GuiMixin, Logger):
     def prevent_timeouts(self):
         self.last_operation = float('inf')
 
+    @runs_in_hwd_thread
     def timeout(self, cutoff):
         '''Time out the client if the last operation was before cutoff.'''
         if self.last_operation < cutoff:
@@ -153,8 +156,9 @@ class SafeTClientBase(HardwareClientBase, GuiMixin, Logger):
 
     @staticmethod
     def expand_path(n):
-        return convert_bip32_path_to_list_of_uint32(n)
+        return convert_bip32_strpath_to_intpath(n)
 
+    @runs_in_hwd_thread
     def cancel(self):
         '''Provided here as in keepkeylib but not safetlib.'''
         self.transport.write(self.proto.Cancel())
@@ -162,6 +166,7 @@ class SafeTClientBase(HardwareClientBase, GuiMixin, Logger):
     def i4b(self, x):
         return pack('>I', x)
 
+    @runs_in_hwd_thread
     def get_xpub(self, bip32_path, xtype):
         address_n = self.expand_path(bip32_path)
         creating = False
@@ -173,6 +178,7 @@ class SafeTClientBase(HardwareClientBase, GuiMixin, Logger):
                          fingerprint=self.i4b(node.fingerprint),
                          child_number=self.i4b(node.child_num)).to_xpub()
 
+    @runs_in_hwd_thread
     def toggle_passphrase(self):
         if self.features.passphrase_protection:
             self.msg = _("Confirm on your {} device to disable passphrases")
@@ -181,14 +187,17 @@ class SafeTClientBase(HardwareClientBase, GuiMixin, Logger):
         enabled = not self.features.passphrase_protection
         self.apply_settings(use_passphrase=enabled)
 
+    @runs_in_hwd_thread
     def change_label(self, label):
         self.msg = _("Confirm the new label on your {} device")
         self.apply_settings(label=label)
 
+    @runs_in_hwd_thread
     def change_homescreen(self, homescreen):
         self.msg = _("Confirm on your {} device to change your home screen")
         self.apply_settings(homescreen=homescreen)
 
+    @runs_in_hwd_thread
     def set_pin(self, remove):
         if remove:
             self.msg = _("Confirm on your {} device to disable PIN protection")
@@ -198,6 +207,7 @@ class SafeTClientBase(HardwareClientBase, GuiMixin, Logger):
             self.msg = _("Confirm on your {} device to set a PIN")
         self.change_pin(remove)
 
+    @runs_in_hwd_thread
     def clear_session(self):
         '''Clear the session to force pin (and passphrase if enabled)
         re-entry.  Does not leak exceptions.'''
@@ -209,10 +219,12 @@ class SafeTClientBase(HardwareClientBase, GuiMixin, Logger):
             # If the device was removed it has the same effect...
             self.logger.info(f"clear_session: ignoring error {e}")
 
+    @runs_in_hwd_thread
     def get_public_node(self, address_n, creating):
         self.creating_wallet = creating
         return super(SafeTClientBase, self).get_public_node(address_n)
 
+    @runs_in_hwd_thread
     def close(self):
         '''Called when Our wallet was closed or the device removed.'''
         self.logger.info("closing client")
