@@ -99,11 +99,13 @@ class Plugin(TrustedCoinPlugin):
                 _('Therefore, two-factor authentication is disabled.')
             ])
             action = lambda: window.show_message(msg)
+            icon = read_QIcon("trustedcoin-status-disabled.png")
         else:
             action = partial(self.settings_dialog, window)
-        button = StatusBarButton(read_QIcon("trustedcoin-status.png"),
-                                 _("TrustedCoin"), action)
-        window.statusBar().addPermanentWidget(button)
+            icon = read_QIcon("trustedcoin-status.png")
+        sb = window.statusBar()
+        button = StatusBarButton(icon, _("TrustedCoin"), action, sb.height())
+        sb.addPermanentWidget(button)
         self.start_request_thread(window.wallet)
 
     def auth_dialog(self, window):
