@@ -165,7 +165,7 @@ class SimpleConfig(Logger):
         try:
             json.dumps(key)
             json.dumps(value)
-        except:
+        except Exception:
             self.logger.info(f"json error: cannot save {repr(key)} ({repr(value)})")
             return
         self._set_key_in_user_config(key, value, save)
@@ -674,22 +674,31 @@ class SimpleConfig(Logger):
         if text:
             try:
                 return NetAddress.from_string(text)
-            except:
+            except Exception:
                 pass
 
-    def format_amount(self, x, is_diff=False, whitespaces=False):
+    def format_amount(
+        self,
+        amount_sat,
+        *,
+        is_diff=False,
+        whitespaces=False,
+        precision=None,
+    ) -> str:
+        if precision is None:
+            precision = self.amt_precision_post_satoshi
         return format_satoshis(
-            x,
+            amount_sat,
             num_zeros=self.num_zeros,
             decimal_point=self.decimal_point,
             is_diff=is_diff,
             whitespaces=whitespaces,
-            precision=self.amt_precision_post_satoshi,
+            precision=precision,
             add_thousands_sep=self.amt_add_thousands_sep,
         )
 
-    def format_amount_and_units(self, amount):
-        return self.format_amount(amount) + ' '+ self.get_base_unit()
+    def format_amount_and_units(self, *args, **kwargs) -> str:
+        return self.format_amount(*args, **kwargs) + ' ' + self.get_base_unit()
 
     def format_fee_rate(self, fee_rate):
         return format_fee_satoshis(fee_rate/1000, num_zeros=self.num_zeros) + ' sat/byte'
@@ -717,8 +726,8 @@ def read_user_config(path):
         with open(config_path, "r", encoding='utf-8') as f:
             data = f.read()
         result = json.loads(data)
-    except:
-        _logger.warning(f"Cannot read config file. {config_path}")
+    except Exception as exc:
+        _logger.warning(f"Cannot read config file at {config_path}: {exc}")
         return {}
     if not type(result) is dict:
         return {}
